# ***********************************************************************************************************
#          CREATION DE FICHIERS DE DIRECTIVES POUR EXTRACTIONS AU FORMAT BUFR, NETCDF & HDF5 (BDMO).
# ===========================================================================================================
#  But    : ecrit des fichiers de directives et effectue des extractions de donnees contenues dans la BDMO
#           au moyen de l'outil 'bdmo_lecture'.
#  Entree - fichier de description des extractions a realiser.           
#  Sortie - fichiers de la forme BUFR.xxx, NETCDF.xxxnnn, HDF5.xxxnnn et REFDATA.xxx pour chaque extraction,
#         - fichier de la forme BATORMAP.xxx pour chaque extraction,
#         - le listing d'execution OULOUTPUT (tee a partir du shell),
#         - le fichier SORTIE, qui donne la liste des suffixes des fichiers BUFR, NETCDF & HDF5 extraits,
#         - les fichiers temporaires DIRECTIVE et tmp_Bufr_99999.
# -----------------------------------------------------------------------------------------------------------
#  auteur                : F. GUILLAUME (CNRM/GMAP/OBS - tel 8467)
#  version               : 1.0
#  date de creation      : 14/09/2005
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 28/09/2005
#  auteur de la revision : F. GUILLAUME
#                          modification des messages d'information, de warning et d'erreur envoyes a l'ecran
#                          et dans OULOUTPUT.
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 11/05/2006
#  auteur de la revision : F. GUILLAUME
#                          - rajout de la possibilite d'extraire un meme type de donnees en plusieurs passes
#                            en reconstituant au final un fichier de donnees unique.
#                          - enrichissement du contenu du fichier ouloutput.
#                          - configuration en fonction du contexte specifie par la variable EXTR_ENV.
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 03/04/2007
#  auteur de la revision : F. GUILLAUME
#                          - rajout de la clef permettant de definir le cutoff.
#                          - rajout des cles qui definissent un domaine geographique precis en utilisant les
#                            nouvelles colonnes de la BDM.
#                          - traitement du cas ou aucune donnee n'est extraite (fichier vide).
#                          - generation optionnelle des fichiers BATORMAP.
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 22/08/2007
#  auteur de la revision : F. GUILLAUME
#                          - prise en charge de la decompression des fichiers gzip des donnees du type RADAR.
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 25/10/2007
#  auteur de la revision : C. PAYAN
#                          - rajout de la possibilite d'extraire les donnees de la base d'INTEGRATION situee
#                            en mode recherche (variable d'environnement SWAPP_BDM_ARCH_ENV).
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 28/02/2008
#  auteur de la revision : F. GUILLAUME
#  version               : 1.1.0
#                          - la generation du fichier BATORMAP est obligatoire.
#                          - rajout de la clef PRECISE OBS DATE pour extraire une donnée à une heure précise
#                            par rapport au pivot .
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 18/02/2008
#  auteur de la revision : F. GUILLAUME
#  version               : 1.1.1
#                          - adaptation a l'environnement SOPRANO.
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 12/08/2008
#  auteur de la revision : F. GUILLAUME
#  version               : 1.1.2
#                          - supression de l'accès aux machine DIAPASON.
#                          - version unique Olive/Chaîne.
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 10/04/2015
#  auteur de la revision : F. GUILLAUME
#  version               : 1.1.3
#                          - changement de la valeur par defaut de MIN INDIC pour prendre en compte certains
#                            messages suite a une erreur de codage dans les indicatifs OMM.
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 05/08/2015
#  auteur de la revision : F. GUILLAUME
#  version               : 1.1.4
#                          - supression de la variable SWAPP_BDM_ARCH_ENV et utilisation des variables
#                            EXTR_ENV et DB_FILE_BDM pour déterminer le contexte d'extraction.
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 24/08/2015
#  auteur de la revision : F. GUILLAUME
#  version               : 1.2.0
#                          - prise en charge des extractions des fichiers au format NETCDF.
#                          - nettoyage.
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 07/10/2015
#  auteur de la revision : F. GUILLAUME
#  version               : 1.2.1
#                          - traitement correct d'une extraction preferentielle des fichiers NETCDF (arome HH
#                            où si absent, HH-15).
#                          - prise en charge d'une seconde formulation pour la clef REFDATA
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 27/06/2016
#  auteur de la revision : F. GUILLAUME
#  version               : 1.2.2
#                          - prise en charge des extractions des fichiers au format HDF5.
#                          - bugfix sur le controle des fichiers absents (sans consequence)
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 25/07/2016
#  auteur de la revision : F. GUILLAUME
#  version               : 1.2.3
#                          - ajout de la compatibilite avec la syntaxe de PostgreSQL ( & Oracle).
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 10/05/2017
#  auteur de la revision : F. GUILLAUME
#  version               : 1.2.4
#                          - ajout de la clef BEFORE EXTENDED pour etendre la borne inferieure de la fenêtre
#                            d'extraction pour les données METEOR. 
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 18/01/2018
#  auteur de la revision : F. GUILLAUME
#  version               : 1.2.5
#                          - bugfix dans le traitement de BEFORE EXTENDED (rajout de before_extend_OK).
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 16/06/2020
#  auteur de la revision : F. GUILLAUME
#  version               : 1.2.6
#                          - bugfix affectant les fichiers batormap lorsque que des fichiers BUFR possèdent 
#                            une racine commune pour leur extension (ex. BUFR.amdar et BUFR.amdaromm)
# -----------------------------------------------------------------------------------------------------------
#  date de revision      : 09/02/2022
#  auteur de la revision : F. GUILLAUME
#  version               : 1.2.7
#                          - adaptation au passage à CentOs-8 : dmtgetdutil est remplacé par opedates
# ***********************************************************************************************************

BEGIN {
  FS                = ":" ;
  RS                = "\n" ;
  OFS               = ":" ;
  ORS               = "\n" ;

  version           = "1.2.6" ;
  contexte          = "OPERATIONNEL" ; # contexte d'execution par defaut.
  extract_soprano   = "bdmo_lecture" ; # fonction d'extraction.
  base              = "" ;

  date_pivot        = 0 ;              # Date pivot.
  fic_sortie_prov   = "SORTIE_TEMP"    # nom du fichier de sortie avant trie.
  fic_sortie        = "SORTIE" ;       # nom du fichier de sortie : liste des OBS.
  fic_directive     = "DIRECTIVE" ;    # nom du fichier de directives d'extraction.
  struct_1_OK       = 0 ;              # permet de savoir si l'on est dans une structure <extraction> .
  struct_2_OK       = 0 ;              # permet de savoir si l'on est dans une structure <heading column>.
  struct_3_OK       = 0 ;              # permet de savoir si l'on est dans une structure <data column>.
  struct_4_OK       = 0 ;              # permettant de savoir si l'on est dans une structure <batormap>.
  before_extend_OK  = 0 ;              # permet de savoir si BEFORE EXTENDED est present dans la requete.
  nb_extract_valid  = 0 ;              # nombre d'extractions avec fichier.
  nb_extract_vide   = 0 ;              # nombre d'extractions vides.
  nb_extract_sans   = 0 ;              # nombre d'extractions sans fichier.
  nb_extract_total  = 0 ;              # nombre d'extractions totales.
  nb_extract_error  = 0 ;              # nombre d'extractions erronnees.
  nb_extract_conc   = 0 ;              # nombre de concatenations.
  nb_extract_fina   = 0 ;              # nombre de fichiers resultant.

# Tableaux donnant le libelle des cles (parametres) attendues en entree et la correspondance eventuelle avec
# celles utilisees dans le fichier de directive.

  gmap["OBS TYPE"]          = "seq_type" ;          # type d'observation BDM.
   
  gmap["UPPER LATITUDE"]    = "max_lat" ;           # latitudes maximale, minimale, longitudes minimale,
  gmap["LOWER LATITUDE"]    = "min_lat" ;           # maximale de la zone demandee en degres decimaux.
  gmap["WESTER LONGITUDE"]  = "min_lon" ;           # SI LE PREMIER SUBSET DU FICHIER N'EST PAS DANS LE
  gmap["EASTER LONGITUDE"]  = "max_lon" ;           # DOMAINE, LE FICHIER NE SERA PAS EXTRAIT.

  gmap["PRECISE UPPER LATITUDE"]   = "PNLAT" ;      # coordonnees precises de la zone de donnees a extraire
  gmap["PRECISE LOWER LATITUDE"]   = "PSLAT" ;      # exprimees en degres decimaux. SI AU MOINS UNE DONNEES
  gmap["PRECISE WESTER LONGITUDE"] = "PWLON" ;      # SI AU MOINS UNE DONNEE DU BUFR EST DANS LE DOMAINE, LE
  gmap["PRECISE EASTER LONGITUDE"] = "PELON" ;      # FICHIER BUFR EST EXTRAIT.

  gmap["MIN INDIC"]         = "min_lltbufr_val" ;   # 1er et dernier indicatifs desires, intervalle max :
  gmap["MAX INDIC"]         = "max_lltbufr_val" ;   # [0,zzzzzzzzzz].

  gmap["SQL FROM"]          = "from_clause";        # clause from.
  gmap["SQL WHERE"]         = "where_clause";       # clause where (/colonnes).
  gmap["SQL ORDER BY"]      = "order_by_clause" ;   # clause de tri SQL.
  gmap["RIGHTS"]            = "code_autorisation" ; # constante  = "OPER".
  gmap["LAST ARRIVED"]      = "last_arrived" ;      # booleen pour obtenir la derniere version valide du 
                                                    # message d'obs ("Y").

  gmap["ARCHIVED DATA"]     = "all_base" ;          # booleen pour avoir acces a la BDM archivee.

  gmap["BINARY OUTPUT"]     = "bitstream" ;         # booleen pour activer l'ecriture du fichier BUFR/NETCDF.

  gmap["ASCII COMMENTED"]   = "imp_commente";       # booleen pour activer l'ecriture du fichier ASCII
                                                    # commente. (fichier inexploitable par programme).

  gmap["ASCII FILENAME"]    = "nom_fich" ;          # nom du fichier ASCII.
  gmap["BINARY FILENAME"]   = "nom_fich_buf" ;      # nom du fichier BUFR.
  gmap["SEPARATOR"]         = "separateur" ;        # caractere separateur dans le fichier ASCII.   

  gmap["PRINT MISSING VAL"] = "imp_val_manq" ;      # booleen pour activer l'impression des valeurs absentes
                                                    # dans le fichier ASCII.

  gmap["ASCII OUTPUT"]      = "ASCII OUTPUT";       # booleen pour activer l'ecriture du fichier ASCII.
                                                    # CETTE CLEF NE SE RETROUVE PAS DANS LE FICHIER FINAL.

  gmap["CUTOFF"]            = "CUTOFF" ;            # date du CUTOFF : format AAAAMMJJhhmmss. CETTE CLEF EST
                                                    # UTILISEE DANS LA CLAUSE WHERE DE LA REQUETE.
						     # where_clause
  gmap["DELTA BEFORE"]      = "DELTA BEFORE" ;      # expressions definissant la fenetre temporelle valide
  gmap["DELTA AFTER"]       = "DELTA AFTER" ;       # des observations  au format "Metronome". CES CLES NE
  gmap["BEFORE EXTENDED"]   = "BEFORE EXTENDED" ;   # SE RETROUVENT PAS DANS LE FICHIER FINAL.



  gmap["CONCATENATION"]    = "CONCATENATION" ;      # booleen pour activer la concatenation des fichiers
                                                    # individuels. Elle est inactive pour les donnees RADAR
                                                    # et les fichiers NETCDF. CETTE CLEF NE SE RETROUVE PAS
                                                    # DANS LE FICHIER FINAL.

  gmap2["OUTPUT"]           = "impression" ;        # booleen pour activer l'ecriture des fichiers de sortie.
 
  gmap2["HEADING COL NR"]   = "nb_col_entete" ;     # nombre de colonnes oracle 'texte' desirees dans le
                                                    # fichier ASCII.

  gmap2["HEADING COL NAME"] = "col_name_entete" ;   # libelle de la colonne oracle 'texte' desiree.

  gmap2["HEADING COL FMT"]  = "col_format_entete" ; # format de la colonne oracle 'texte' desiree.

  gmap2["BATORMAP ENTRY NR"]= "BATORMAP ENTRY NR" ; # nombre d'entree dans le fichier batormap pour le
                                                    # fichier traite.

  gmap2["TARGET BASE"]      = "TARGET BASE" ;       # base ECMA de destination pour le capteur.

  gmap2["SENSOR"]           = "SENSOR" ;            # capteur concerne.

  gmap2["DATA COL NR"]      = "nb_col" ;            # nombre de colonnes oracle 'donnees' desirees dans le
                                                    # fichier ASCII.

  gmap2["DATA COL NAME"]    = "col_name" ;          # libelle de la colonne oracle 'donnees' desiree.

  gmap2["DATA COL FMT"]     = "col_format" ;        # format de la colonne oracle 'donnee' desiree.

  gmap2["TEMPO FILE"]       = "TEMPO FILE" ;        # format de la colonne oracle 'donnee' desiree. CETTE
                                                    # CLEF NE SE RETROUVE PAS DANS LE FICHIER FINAL.

  gmap2["DATA COL MISSING"] = "col_manq" ;          # valeur manquante pour la colonne oracle 'donnee' desiree.

  gmap["PRECISE OBS DATE"]  = "PRECISE OBS DATE" ;  # date precise de l'observation au format "Metronome" par
                                                    # rapport à la date pivot.

# recuperation du contexte d'execution du script.
  contexte = ENVIRON["EXTR_ENV"] ;
  base     = ENVIRON["DB_FILE_BDM"] ;
						

# recuperation et controle de la date pivot. Si elle est incorrecte, on sort du script.
  print "=== DEBUT DE PROGRAMME D'EXTRACTION ===" ;
  printf("               v. %s\n",version) ;
  date_pivot = ENVIRON["DMT_DATE_PIVOT"] ;
  if (length(date_pivot) != 14 || match(date_pivot,/[^0-9]/) != 0) {
    print "*** ERREUR  : recuperation date pivot impossible" ;
    exit ;
  }
  printf("===     POUR LE %14s.     ===\n\n",date_pivot) ;
}


# Un bloc d'extraction est delimite par les tags <EXTRACTION> et </EXTRACTION> dans le fichier d'entree.
# En entrant dans <EXTRACTION>, on initialise les valeurs par defaut associees aux differentes cles.
toupper($0) ~ /<EXTRACTION>/ && NF == 1 {
  val[gmap["OBS TYPE"]]                 = "" ;
  val[gmap["UPPER LATITUDE"]]           =   90.00000 ;
  val[gmap["LOWER LATITUDE"]]           =  -90.00000 ;
  val[gmap["WESTER LONGITUDE"]]         = -180.00000 ;
  val[gmap["EASTER LONGITUDE"]]         =  180.00000 ;
  val[gmap["MIN INDIC"]]                = "0" ;
  val[gmap["MAX INDIC"]]                = "zzzzzzzzzz" ;
  val[gmap["SQL FROM"]]                 = "" ;
  val[gmap["SQL WHERE"]]                = "" ;
  val[gmap["SQL ORDER BY"]]             = "" ;
  val[gmap["RIGHTS"]]                   = "OPER" ;
  val[gmap["PRECISE UPPER LATITUDE"]]   = "NULL" ;
  val[gmap["PRECISE LOWER LATITUDE"]]   = "NULL" ;
  val[gmap["PRECISE WESTER LONGITUDE"]] = "NULL" ;
  val[gmap["PRECISE EASTER LONGITUDE"]] = "NULL" ;
  if (contexte == "RECHERCHE") {   
    if (base == "/usr/local/sopra/neons_db_bdm.archi") {
      val[gmap["ARCHIVED DATA"]]        = "Y" ;     # on accede a la BDM archive et on extrait tous
      val[gmap["LAST ARRIVED"]]         = "N" ;     # les avenants des messages.
    } else {
      val[gmap["ARCHIVED DATA"]]        = "N" ;     # on accede a la BDM en ligne et on extrait tous
      val[gmap["LAST ARRIVED"]]         = "N" ;     # les avenants des messages.
    }
  } else {
    if (base == "/usr/local/sopra/neons_db_bdm.archi") {
      val[gmap["ARCHIVED DATA"]]        = "Y" ;     # on accede a la BDM archive et on extrait que
      val[gmap["LAST ARRIVED"]]         = "Y" ;     # le dernier avenant des messages.
    } else {
      val[gmap["ARCHIVED DATA"]]        = "N" ;     # on accede a la BDM en ligne et on extrait que
      val[gmap["LAST ARRIVED"]]         = "Y" ;     # le dernier avenant des messages.
    }
  }
  val[gmap["BINARY OUTPUT"]]            = "N" ;
  val[gmap["ASCII FILENAME"]]           = "/dev/null" ;
  val[gmap["BINARY FILENAME"]]          = "/dev/null" ;
  val[gmap["ASCII COMMENTED"]]          = "N" ; 
  val[gmap["SEPARATOR"]]                = "," ;
  val[gmap["PRINT MISSING VAL"]]        = "N" ;
  val[gmap["ASCII OUTPUT"]]             = "N" ;
  val[gmap["CUTOFF"]]                   = ""
  val[gmap["DELTA BEFORE"]]             = "-3hours +%Y%m%d%H%M%S" ;
  val[gmap["DELTA AFTER"]]              = "+2hours +59minutes +59seconds +%Y%m%d%H%M%S" ;
  val[gmap["BEFORE EXTENDED"]]          = val[gmap["DELTA BEFORE"]] ;

  val[gmap["CONCATENATION"]]            = "N" ;
  val[gmap2["TEMPO FILE"]]              = "tmp_Bufr_99999" ;
  val[gmap2["OUTPUT"]]                  = "N" ;
  val[gmap2["HEADING COL NR"]]          = 0 ;
  val[gmap2["DATA COL NR"]]             = 0 ;
  val[gmap2["BATORMAP ENTRY NR"]]       = 0 ;
  val[gmap["PRECISE OBS DATE"]]         = "NULL" ;
  if ((struct_1_OK + struct_2_OK + struct_3_OK + struct_4_OK) == 0) {
    struct_1_OK                         = 1 ;       # on rentre dans une structure <extraction>.
    refdata_OK                          = 0 ;       # pour l'instant pas encore de REFDATA
  } else {
    printf("*** ERREUR : tag <extraction> imbrique dans '%s'\n",FILENAME) ;
    exit 180;
  }
}


# recuperation d'un couple clef-valeur d'une structure <extraction>.
NF == 2 && struct_1_OK == 1 && (struct_2_OK + struct_3_OK + struct_4_OK) == 0 {
  cle            = sansblanc(toupper($1)) ;
  valeur         = sansblanc($2) ;
  val[gmap[cle]] = valeur ;
  if (cle == "BEFORE EXTENDED") {
    before_extend_OK = 1 ;
  }
}


# si l'enregistrement comporte 3 champs, c'est la description d'un fichier REFDATA qu'on recupere.
NF == 3 && struct_1_OK == 1 && (struct_2_OK + struct_3_OK + struct_4_OK) == 0 &&
           sansblanc(toupper($1)) == "REFDATA" {
  refdata_fmt     = sansblanc($2) ;
  refdata_date    = substr(date_pivot,1,8) ;
  refdata_res     = substr(date_pivot,9,2) ;
  refdata_inst    = sansblanc($3) ;
  refdata_nb      = 1 ;
  if (length(refdata_fmt)*length(refdata_inst) > 0) {
    refdata_OK    = 1 ;
  }
}

# si l'enregistrement comporte 4 champs, c'est la description d'un fichier REFDATA
# ancien format qu'on récupere (compatibilite ascendante).
NF == 4 && struct_1_OK == 1 && (struct_2_OK + struct_3_OK + struct_4_OK) == 0 &&
           sansblanc(toupper($1)) == "REFDATA" {
  refdata_fic     = sansblanc($2) ;
  refdata_fmt     = sansblanc($3) ;
  refdata_date    = substr(date_pivot,1,8) ;
  refdata_res     = substr(date_pivot,9,2) ;
  refdata_inst    = sansblanc($4) ;
  refdata_nb      = 1 ;
  if (length(refdata_fic)*length(refdata_fmt)*length(refdata_inst) > 0) {
    refdata_OK    = 1 ;
  }
}

# Quand on sort d'une description d'extraction </EXTRACTION> on ecrit eventuellement
# un fichier de directive, on execute la demande d'extraction et on controle son resultat.
toupper($0) ~ /<\/EXTRACTION>/ && (NF * struct_1_OK == 1) {
  erreur_date_min   = 0 ;         # code retour de 'opedates' date debut
  erreur_date_max   = 0 ;         # code retour de 'opdates' date fin
  erreur_date_exten = 0 ;         # code retour de 'opedates' date exten

  erreur_bdm_lect   = 0 ;         # code retour de 'bdm_lecture'
  erreur_fic_vide   = 0 ;         # code retour de fichier de donnees vide
  erreur_concaten   = 0 ;         # code retour de 'concatenation de fichiers'
  erreur_gzip       = 0 ;         # code retour de gzip -d pour les RADAR
  on_traite         = 1 ;         # si 1 la requete sera traitee.
  struct_1_OK       = 0 ;         # on sort d'une structure extraction.

  get_date_exten    = "opedates "date_pivot" "val[gmap["BEFORE EXTENDED"]] ;
  get_date_min      = "opedates "date_pivot" "val[gmap["DELTA BEFORE"]] ;
  get_date_max      = "opedates "date_pivot" "val[gmap["DELTA AFTER"]] ;
  get_date_obs      = "opedates "date_pivot" "val[gmap["PRECISE OBS DATE"]] ;
  launch_extraction = extract_soprano " " fic_directive ;
  if (refdata_fmt == "NETCDF" || refdata_fmt == "HDF5") {
    liste_de_fichiers = "tempo=`ls "val[gmap["BINARY FILENAME"]]"*` ; echo $tempo" ;
  } else {
    liste_de_fichiers = "tempo=`ls "val[gmap["BINARY FILENAME"]]"` ; echo $tempo" ;
  }
  printf("\n---> tentative d'extraction pour '%s' sur %s\n",val[gmap["OBS TYPE"]],base) ;

  if (val[gmap["CONCATENATION"]] == "Y" && (val[gmap["OBS TYPE"]] == "RADAR" || refdata_fmt == "NETCDF" || refdata_fmt == "HDF5")) {
    printf("*** WARNING : concatenation interdite et ignoree pour le type %s\n",val[gmap["OBS TYPE"]]) ;
    val[gmap["CONCATENATION"]] = "N" ;
  }
  if (val[gmap["CONCATENATION"]] == "Y") {
    ctrl_vide  =  "test -s "val[gmap2["TEMPO FILE"]] ;
    menage     = "rm "val[gmap2["TEMPO FILE"]] ;
  } else {
    ctrl_vide  = "err=2 ; if [ $(ls "val[gmap["BINARY FILENAME"]]"* 2>>/dev/null | wc -l) -gt 0 ] ; then for i in `ls "\
                 val[gmap["BINARY FILENAME"]]"*`; do if [ -s $i ] ; then err=0 ; fi; done; fi ; exit $err" ;
    menage     = "rm `ls "val[gmap["BINARY FILENAME"]]"*`" ;
  }
  concatene1   = "cat "val[gmap2["TEMPO FILE"]]" >"val[gmap["BINARY FILENAME"]] ;
  concatene2   = "cat "val[gmap2["TEMPO FILE"]]" >>"val[gmap["BINARY FILENAME"]] ;

  erreur_date_min = (get_date_min) | getline val["min_date"] ;
  close(get_date_min) ;
  if (erreur_date_min != 1) {
    printf("*** WARNING : '%s' incorrect pour DELTA BEFORE.\n",val[gmap["DELTA BEFORE"]]) ;
    on_traite       = 0 ;
  }
  erreur_date_max = get_date_max | getline val["max_date"] ;
  close(get_date_max) ;
  if (erreur_date_max != 1) {
    printf("*** WARNING : '%s' incorrect pour DELTA AFTER.\n",val[gmap["DELTA AFTER"]]) ;
    on_traite       = 0 ;
  }
  erreur_date_exten = get_date_exten | getline val["ext_date"] ;
  close(get_date_exten) ;
  if (erreur_date_exten != 1) {
    printf("*** WARNING : '%s' incorrect pour BEFORE EXTENDED\n",val[gmap["BEFORE EXTENDED"]]) ;
    on_traite       = 0 ;
  } else {
    if (before_extend_OK == 0) {
      val["ext_date"] = val["min_date"] ;
    }
    if (val["min_date"] < val["ext_date"]) {
      val["ext_date"] = val["min_date"]
      printf("*** WARNING : BEFORE EXTENDED replaced with DELTA BEFORE.\n",val[gmap["DELTA BEFORE"]]) ;
    }
  }
  if (val[gmap["PRECISE OBS DATE"]] !="NULL") {
    erreur_date_obs = (get_date_obs) | getline val[gmap["PRECISE OBS DATE"]] ;
    close(get_date_obs) ;
    if (erreur_date_obs != 1) {
      printf("*** WARNING : '%s' incorrect pour PRECISE OBS DATE.\n",val[gmap["PRECISE OBS DATE"]]) ;
      on_traite     = 0 ;
    }
  }
  if (val[gmap["OBS TYPE"]] == "") {
    printf("*** WARNING : '%s' incorrect pour OBS TYPE.\n",val[gmap["OBS TYPE"]]) ;
    on_traite       = 0 ;
  }
  if (refdata_OK != 1) {
    printf( "*** WARNING : REFDATA non renseigne ou incorrect pour %s.\n",val[gmap["OBS TYPE"]]) ;
    on_traite       = 0 ;
  }
  if (val[gmap2["BATORMAP ENTRY NR"]] < 1) {
    printf( "*** WARNING : BATORMAP non renseigne ou incorrect pour %s.\n",val[gmap["OBS TYPE"]]) ;
    on_traite       = 0 ;
  }
  nb_extract_total++ ;

# Si on a un type BDM, un intervalle de date et un refdata valables lance une extraction.
  if (on_traite ==  1) { 
    if (val["ext_date"] != val["min_date"]) {
      val["where_clause"] = val["where_clause"] " and "\
      "(((cast(jour_max as bigint)*1000000+cast(heure_max as bigint)) between "val["min_date"]" and "val["max_date"]") "\
      "or ((cast(jour_min as bigint)*1000000+cast(heure_min as bigint)) between "val["min_date"]" and "val["max_date"]"))" ;
      val["min_date"] = val["ext_date"] ;
    }
    if (val[gmap["PRECISE OBS DATE"]] != "NULL") {
      val["where_clause"] = val["where_clause"] " and "\
          "dat = to_timestamp('"val["PRECISE OBS DATE"] "','YYYYMMDDHH24MISS')" ;
    }
    if (length(val["CUTOFF"]) != 14 && val["CUTOFF"] != "") {
      printf( "*** WARNING : CUTOFF '%s' incorrect et ignore.\n",val["CUTOFF"]) ;
    } else {
      if (val["CUTOFF"] != "") val["where_clause"] = val["where_clause"] " and "\
         "date_insert < to_timestamp('"val["CUTOFF"] "','YYYYMMDDHH24MISS')" ;
    }
    if (match(val["PNLAT"]val["PSLAT"]val["PELON"]val["PWLON"],/NULL/)==0) {
      if (val["PNLAT"]-0.<val["PSLAT"]-0. || val["PNLAT"]-90>0. || val["PSLAT"]+90<0. ||
          val["PELON"]-180>0. || val["PWLON"]-180>0. || val["PELON"]+180<0. || val["PWLON"]+180<0.) {
        print "*** WARNING : Domaine Lat/Lon Precis incorrect donc ignore.\n" ;
      } else {
        val["where_clause"] = val["where_clause"] " and ((((" val["PELON"] ">=" val["PWLON"]\
                              " and ((minlon<=maxlon and minlon<=" val["PELON"] " and maxlon>="\
                              val["PWLON"] ") or (minlon>maxlon and (minlon<=" val["PELON"]\
                              " or maxlon>="val["PWLON"] ")))) or (" val["PELON"] "<" val["PWLON"]\
                              " and ((minlon<=maxlon and (minlon<=" val["PELON"] " or maxlon>="\
                              val["PWLON"] ")) or minlon>maxlon))) and (minlat<=" val["PNLAT"]\
                              " and maxlat>=" val["PSLAT"] ")) or minlon is NULL or maxlon is NULL"\
                              " or minlat is NULL or maxlat is NULL)" ;
      }
    }

# Si on a une clause SQL where, elle ne doit pas depasser 900 caracteres.
    if (length(val["where_clause"]) >900 ) {
      print "*** WARNING : clause SQL Where trop longue. Extraction rejetee.\n" ;
      next ;
    }
    directive_prt("20s","seq_type")
    directive_prt("10s","min_lat")
    directive_prt("10s","max_lat")
    directive_prt("10s","min_lon")
    directive_prt("10s","max_lon")
    directive_prt("14s","min_date")
    directive_prt("14s","max_date")
    directive_prt("10s","min_lltbufr_val")
    directive_prt("10s","max_lltbufr_val")
    directive_prt("-126s","from_clause")
    directive_prt("-900s","where_clause")
    directive_prt("-126s","order_by_clause")
    directive_prt("1s","all_base")
    directive_prt("-30s","code_autorisation")
    directive_prt("1s","last_arrived")
    directive_prt("1s","bitstream")

# Si on demande la sortie d'un fichier Binaire ou ASCII on active l'output de bdm_lecture.
    if (val["bitstream"] == "Y" || val["ASCII OUTPUT"] == "Y") val[gmap2["OUTPUT"]] = "Y" ;
    directive_prt("1s","impression")
    directive_prt("-100s","nom_fich")
    if (val[gmap["CONCATENATION"]] == "Y" ) {
      fic_permutation = val[gmap["BINARY FILENAME"]] ;
      val[gmap["BINARY FILENAME"]] = val[gmap2["TEMPO FILE"]] ;
      directive_prt("-100s","nom_fich_buf") ;
      val[gmap["BINARY FILENAME"]] = fic_permutation ;
    } else {
      directive_prt("-100s","nom_fich_buf")
    }
    directive_prt("1s","imp_commente")
    directive_prt("1s","separateur")
    directive_prt("1s","imp_val_manq")

# On gere ici la generation des "colonnes entetes oracle" si demandees.
    if (val["nb_col_entete"] < 0) val["nb_col_entete"] = 0 ;
    directive_prt("3s","nb_col_entete")
    for (i=1;i<=val["nb_col_entete"];i++) {
      directive_multiprt("26s","col_name_entete",i)
      directive_multiprt("3s","col_format_entete",i)
    }

# On gere ici la generation des "colonnes de donnees" si demandees.
    if (val["nb_col"] < 0) val["nb_col"] = 0 ;
    directive_prt("3s","nb_col")
    for (i=1;i<=val["nb_col"];i++) {
      directive_multiprt("26s","col_name",i)
      directive_multiprt("7s","col_format",i)
      directive_multiprt("6s","col_manq",i)
    }
    close(fic_directive) ;

# On lance la requete d'extraction.
    erreur_bdm_lect = system(launch_extraction) ;
    close(launch_extraction) ;
    erreur_fic_vide = system(ctrl_vide) ;
    close(ctrl_vide) ;

# post traitement du resultat de l'extraction demandee.
    if (erreur_bdm_lect != 0 ) {
      print "*** WARNING : extraction incorrecte." ;
      nb_extract_error++ ;
      next ;
    }
    if (erreur_fic_vide != 0 ) {
      print "*** WARNING : fichier vide ou absent." ;
      nb_extract_vide++ ;
      system(menage) ;
      close(menage) ;
      next ;
    }
    if (val["impression"] == "Y" && val["nom_fich_buf"] != "/dev/null") {
      "date +%Y%m%d%H%M%S" | getline cutoff ;
      close("date +%Y%m%d%H%M%S") ;
      if (val[gmap["CONCATENATION"]] == "Y") {
        fic = val[gmap["BINARY FILENAME"]] ;
      } else {
        liste_de_fichiers | getline fic ;
        close(liste_de_fichiers) ;
      }
      split(fic,ficlist," ") ;
      if (contexte != "RECHERCHE") printf("-- cutoff pour %s : %14s\n",val["seq_type"],cutoff) ;
      for (fic in ficlist) {
        ficlist[fic] = substr(ficlist[fic],index(ficlist[fic],".")+1)
        printf("%-8s %-8s %-16s %8s %2s\n",ficlist[fic],refdata_fmt,
               refdata_inst,refdata_date,refdata_res) > "refdata."ficlist[fic] ;
        close("refdata."ficlist[fic]) ;
      }
      if (val["BATORMAP ENTRY NR"] > 0) {
        for (i=1;i<=val["BATORMAP ENTRY NR"];i++) {
          for (fic in ficlist) {
            printf("%-8s %-8s %-8s %-16s\n",val["TARGET BASE"i],ficlist[fic],refdata_fmt,
                   val["SENSOR"i]) > "batormap."ficlist[fic] ;
            close("batormap."ficlist[fic]) ;
          }
        }
      }
      if (val[gmap["CONCATENATION"]] == "Y") {
        if (val[gmap["BINARY FILENAME"]] in compteur) {
          erreur_concaten = system(concatene2) ;
          close(concatene2) ;
          compteur[val[gmap["BINARY FILENAME"]]]++ ;
        } else {
          erreur_concaten = system(concatene1) ;
          close(concatene1) ;
          compteur[val[gmap["BINARY FILENAME"]]] = 1 ;
        }
        if (erreur_concaten != 0) {
          print "*** ERREUR : concatenation impossible pour "val[gmap["BINARY FILENAME"]] ;
          exit ;
        } else {
          print "-- concatenation des "val["seq_type"]" activee dans "val[gmap["BINARY FILENAME"]] ;
          if (compteur[val[gmap["BINARY FILENAME"]]] == 1) {
            for (fic in ficlist) {
              printf("%-8s\n",ficlist[fic]) >fic_sortie_prov ;
            }
            nb_extract_conc++ ;
          } else {
            nb_extract_fina-- ;
          }
          system(menage) ;
          close(menage) ;
        }
      } else {
        if (val[gmap["OBS TYPE"]] == "RADAR") {
          print "---> tentative de decompression du fichier radar." ;
          system("mv "val[gmap["BINARY FILENAME"]]" "val[gmap["BINARY FILENAME"]]".gz") ;
          close("mv "val[gmap["BINARY FILENAME"]]" "val[gmap["BINARY FILENAME"]]".gz") ;
          erreur_gzip = system("gzip -d "val[gmap["BINARY FILENAME"]]".gz") ;
          close("gzip -d "val[gmap["BINARY FILENAME"]]".gz") ;
          if (erreur_gzip != 0){
            print "*** WARNING : erreur de decompression." ;
            nb_extract_error++ ;
            menage_radar = "rm "val[gmap["BINARY FILENAME"]]".gz batormap."ficlist[fic]" refdata."\
                           ficlist[fic] ;
            system(menage_radar) ;
            close(menage_radar) ;
            next ;
          }
        }
        for (fic in ficlist) {
          printf("%-8s\n",ficlist[fic]) >fic_sortie_prov ;
        }
      }
      nb_extract_valid++ ;
      nb_extract_fina++ ;
    } else {
      nb_extract_sans++ ;
    }
  } else {
    printf("*** WARNING : extraction '%s' rejetee car mal renseignee.\n",val[gmap["OBS TYPE"]]) ;
    nb_extract_error++ ;
  }
}


# recuperation d'un couple clef-valeur d'une structure <HEADING COLUMN>. (non utilise en operationnel).
NF == 2 && struct_2_OK == 1 {
  cle    = sansblanc(toupper($1)) ;
  valeur = sansblanc($2) ;
  if (cle == "HEADING COL NAME" && heading_name_OK == 0) {
    val[gmap2[cle]val[gmap2["HEADING COL NR"]]] = valeur ;
    heading_name_OK                             = 1 ;
  }    
  if (cle == "HEADING COL FMT" && heading_fmt_OK == 0) {
    val[gmap2[cle]val[gmap2["HEADING COL NR"]]] = valeur ;
    heading_fmt_OK                              = 1 ;
  }    
}


# recuperation d'un couple clef-valeur d'une structure <DATA COLUMN>. (non utilise en operationnel).
NF == 2 && struct_3_OK == 1 {
  cle    = sansblanc(toupper($1)) ;
  valeur = sansblanc($2) ;
  if (cle == "DATA COL NAME" && data_name_OK == 0) {
    val[gmap2[cle]val[gmap2["DATA COL NR"]]] = valeur ;
    data_name_OK                             = 1 ;
  }    
  if (cle == "DATA COL FMT" && data_fmt_OK == 0) {
    val[gmap2[cle]val[gmap2["DATA COL NR"]]] = valeur ;
    data_fmt_OK                              = 1 ;
  }    
  if (cle == "DATA COL MISSING" && data_missing_OK == 0) {
    val[gmap2[cle]val[gmap2["DATA COL NR"]]] = valeur ;
    data_missing_OK                          = 1 ;
  }    
}


# recuperation d'un couple clef-valeur d'une structure <BATORMAP>.
NF == 2 && struct_4_OK == 1 {
  cle    = sansblanc(toupper($1)) ;
  valeur = sansblanc($2) ;
  if (cle == "TARGET BASE" && target_base_OK == 0 && length(valeur) > 0) {
    val[gmap2[cle]val[gmap2["BATORMAP ENTRY NR"]]] = valeur ;
    target_base_OK                                 = 1 ;
  }    
  if (cle == "SENSOR" && sensor_OK == 0 && length(valeur) > 0) {
    val[gmap2[cle]val[gmap2["BATORMAP ENTRY NR"]]] = valeur ;
    sensor_OK                                      = 1 ;
  }    
}


# On rentre dans une structure <HEADING COLUMN>. (non utilise en operationnel).
toupper($0) ~ /<HEADING COLUMN>/ && (NF * struct_1_OK) == 1 {
  if ((struct_2_OK + struct_3_OK + struct_4_OK) == 0) {
    struct_2_OK     = 1 ;
    heading_name_OK = 0 ;
    heading_fmt_OK  = 0 ;
    val[gmap2["HEADING COL NR"]]++ ;
  } else {
    printf("\n*** ERREUR  : tag <heading column> imbrique dans '%s'\n",FILENAME) ;
    exit 180;
  }
}


# On sort d'une structure <HEADING COLUMN>. (non utilise en operationnel).
toupper($0) ~ /<\/HEADING COLUMN>/ && (NF * struct_1_OK * struct_2_OK) == 1 {
  struct_2_OK = 0 ;
  status      = heading_name_OK * heading_fmt_OK ;
  if (status != 1) val[gmap2["HEADING COL NR"]]-- ;
}


# On rentre dans une structure <BATORMAP>.
toupper($0) ~ /<BATORMAP>/ && (NF * struct_1_OK) == 1 {
  if ((struct_2_OK + struct_3_OK + struct_4_OK) == 0) {
    struct_4_OK     = 1 ;
    target_base_OK  = 0 ;
    sensor_OK       = 0 ;
    val[gmap2["BATORMAP ENTRY NR"]]++ ;
  } else {
    printf("\n*** ERREUR  : tag <batormap> imbrique dans '%s'\n",FILENAME) ;
    exit 180;
  }
}


# On sort d'une structure <BATORMAP>.
toupper($0) ~ /<\/BATORMAP>/ && (NF * struct_1_OK * struct_4_OK) == 1 {
  struct_4_OK = 0 ;
  status      = target_base_OK * sensor_OK ;
  if (status != 1) val[gmap2["BATORMAP ENTRY NR"]]-- ;
}


# On rentre dans une structure <DATA COLUMN>. (non utilise en operationnel).
toupper($0) ~ /<DATA COLUMN>/ && (NF * struct_1_OK) == 1 {
  if ((struct_2_OK + struct_3_OK + struct_4_OK) == 0) {
    struct_3_OK      = 1 ;
    data_name_OK     = 0 ;
    data_fmt_OK      = 0 ;
    data_missing_OK  = 0 ;
    val[gmap2["DATA COL NR"]]++ ;
  } else {
    printf("\n*** ERREUR  : tag <data column> imbrique dans '%s'\n",FILENAME) ;
    exit 180;
  }
}


# On sort d'une structure <DATA COLUMN>. (non utilise en operationnel).
toupper($0) ~ /<\/DATA COLUMN>/ && (NF * struct_1_OK * struct_3_OK) == 1 {
  struct_3_OK = 0 ;
  status      = data_name_OK * data_fmt_OK * data_missing_OK ;
  if (status != 1) val[gmap2["DATA COL NR"]]-- ;
}


END {
  if (nb_extract_total == 0) {
    print "\n*** WARNING : aucune extraction trouvee !" ;
  } else {
    printf("\n--- Nombre d'extractions demandees    : %2i\n",nb_extract_total) ;
    printf("----------   extractions erronees     : %2i\n",nb_extract_error) ;
    printf("----------   extractions fichier vide : %2i\n",nb_extract_vide) ;
    printf("----------   extractions sans fichier : %2i\n\n",nb_extract_sans) ;
    printf("----------   extractions avec fichier : %2i\n\n",nb_extract_valid) ;
    printf("--- Nombre de concatenations          : %2i\n\n",nb_extract_conc) ;
    printf("--- Nombre de fichiers resultants     : %2i\n",nb_extract_fina) ;
    close(fic_sortie_prov) ;
    menage = "if [ -s "fic_sortie_prov" ] ; then sort -u "fic_sortie_prov" >"fic_sortie" ; fi ; rm "\
             fic_sortie_prov" 2>>/dev/null";
    system(menage) ;
    close(menage) ;
  }
  print "\n===  FIN DE PROGRAMME D'EXTRACTION  ===" ;
}


# fonction de supression de blanc.
function sansblanc(chaine) {
  sub(/^( )*/,"",chaine) ;
  sub(/( )*$/,"",chaine) ;
  return (chaine) ;
}

# fonction d'impression "multiple" dans le fichier de directive (colonnes).
function directive_multiprt(format,clef,ind) {
  fmt  = clef":%"format"\n" ;
  printf(fmt,val[clef ind]) > fic_directive ;
  return ;
}

# fonction d'inpression dans le fichier de directive.
function directive_prt(format,clef) {
  fmt = clef":%"format"\n" ;
  printf(fmt,val[clef]) > fic_directive ;
  return ;
}
